// Dib.h: interface for the CDib class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DIB_H__836A3825_8886_11D2_AFC1_F5CC710F986B__INCLUDED_)
#define AFX_DIB_H__836A3825_8886_11D2_AFC1_F5CC710F986B__INCLUDED_

#include "MSU_Structs.h"
#include "MSU_ImageBuffer.h"
 
namespace MSU
{

class IProgressBar;

////////////////////////////////////////////////////////////////////////////////////////////
//  CDib24
//	Wrap class for images in BMP 24
//
int CalcDib24WidthInBytes(int Width);

class CDib24 : public CImageBuffer<TRGBColor, CalcDib24WidthInBytes>
{
public:
	// constructs an empty image. (Use Create or Load methods after)
	CDib24() {};

	// constructs a new image from the buffer
	CDib24(TRGBColor* p, const BITMAPINFOHEADER &info, bool bOwn): 
		CImageBuffer<TRGBColor, CalcDib24WidthInBytes>(CSize(info.biWidth, info.biHeight), p, bOwn), m_Info(info) {}

	// constructs a new subimage (Size=(-1, -1) - the whole image)
	CDib24(const CDib24& Im, const CPoint &OriginPt=CPoint(0, 0), const CSize& Size=CSize(-1, -1), bool bOwn=true):
		CImageBuffer<TRGBColor, CalcDib24WidthInBytes>(Im, OriginPt, Size, bOwn) { 	PrepareInfoHeader(Size.cx==-1?Im.Size():Size); }

public:	
	// Creates / Destroys image
	bool Create(const CSize &size);
	void Destroy();

public:
	// Loads / Saves image
	bool Load(const char* FileName, IProgressBar *pProgressBar=NULL);
	bool Save(const char* FileName, IProgressBar *pProgressBar=NULL);

public:
	// Draws image
	void Draw(HDC hdc, int x=0, int y=0, int destWidth=0, int destHeight=0, int srcX=0, int srcY=0, int srcWidth=0, int srcHeight=0);

public:
	BITMAPINFOHEADER &DibInfo() { _ASSERT(m_Ptr); return m_Info; }

protected:
	void PrepareInfoHeader(const CSize& size);
	BITMAPINFOHEADER m_Info;
};


void LoadBMP(std::ifstream &file, BITMAPINFO* &pDIBInfo, void* &pDIBBits, 
	IProgressBar *pProgressBar); //	throw MSU::EMSUException;

void SaveBMP(std::ofstream &file, BITMAPINFO* pDIBInfo, void* pDIBBits, 
	IProgressBar *pProgressBar); //	throw MSU::EMSUException;



}; // namespace MSU

#endif // !defined(AFX_DIB_H__836A3825_8886_11D2_AFC1_F5CC710F986B__INCLUDED_)
