#include "stdafx.h"

#include <crtdbg.h>
#include <math.h>
#include "MSU_Matrix.h"

namespace MSU
{
const CVect CVect::O(0, 0);
const CVect3 CVect3::O(0, 0, 0);
const CMatrix CMatrix::E(1, 0, 0, 1);
const CGenMatrix CGenMatrix::E(CMatrix::E, CVect::O);

double Norm(const CVect& v) { return sqrt(v.x*v.x + v.y*v.y); }

double Norm(const CVect3& v) { return sqrt(v.x*v.x + v.y*v.y + v.z*v.z); }


CMatrix Inverse(const CMatrix& v) 
{ 
	CMatrix v1(v);	
	bool bOk = v1.Invert();	_ASSERT(bOk);
	return v1;
}

CGenMatrix Inverse(const CGenMatrix& v)
{
	CGenMatrix v1(v);	
	bool bOk = v1.Invert();	_ASSERT(bOk);
	return v1;
}

// blend M(t): E->M, 0<=t<=1
CGenMatrix Blend(const CGenMatrix& m, double t)
{
	_ASSERT(t>=0 && t<=1);
	double t1=1-t;
	CGenMatrix m1(t1+m.A.a11*t, m.A.a12*t, m.A.a21*t, t1+m.A.a22*t, m.b.x*t, m.b.y*t);
	return m1;
}


#define STREAM_ASSERT(b, str)\
	if (!str)\
		return str;\
	if (!(b))\
	{\
		str.setstate(std::ios::failbit);\
		return str;\
	}

std::istream &operator>>(std::istream &str, CVect &v)
{
	char sep; str >> std::ws >> sep;	STREAM_ASSERT(sep=='{', str);
	str >> v.x >> std::ws >> sep;		STREAM_ASSERT(sep==',', str);
	str >> v.y >> std::ws >> sep;		STREAM_ASSERT(sep=='}', str);
	return str;
}

std::istream &operator>>(std::istream &str, CVect3 &v)
{
	char sep; str >> std::ws >> sep;	STREAM_ASSERT(sep=='{', str);
	str >> v.x >> std::ws >> sep;		STREAM_ASSERT(sep==',', str);
	str >> v.y >> std::ws >> sep;		STREAM_ASSERT(sep==',', str);
	str >> v.z >> std::ws >> sep;		STREAM_ASSERT(sep=='}', str);
	return str;
}

std::istream &operator>>(std::istream &str, CMatrix &m)
{
	char sep; str >> std::ws >> sep;	STREAM_ASSERT(sep=='{', str);
	str >> m.a11 >> std::ws >> sep;		STREAM_ASSERT(sep==',', str);
	str >> m.a12 >> std::ws >> sep;		STREAM_ASSERT(sep==',', str);
	str >> m.a21 >> std::ws >> sep;		STREAM_ASSERT(sep==',', str);
	str >> m.a22 >> std::ws >> sep;		STREAM_ASSERT(sep=='}', str);
	return str;
}

std::istream &operator>>(std::istream &str, CGenMatrix &m)
{
	str >> m.A >> m.b;
	return str;
}


std::ostream &operator<<(std::ostream &str, const CVect &v)
{
	str << "{ " << v.x << ", " << v.y << " }" << std::endl;
	return str;
}

std::ostream &operator<<(std::ostream &str,	const CVect3 &v)
{
	str << "{ " << v.x << ", " << v.y << ", " << v.z << " }" << std::endl;
	return str;
}

std::ostream &operator<<(std::ostream &str, const CMatrix &m)
{
	str << "{ " << m.a11 << ", " << m.a12 << ", " << m.a21 << ", " << m.a22 << " }" << std::endl;
	return str;
}

std::ostream &operator<<(std::ostream &str, const CGenMatrix &m)
{
	str << m.A << m.b;
	return str;
}



}// namespace MSU

