// utils.h: interface for the utils class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_UTILS_H__0A7298A9_87BA_11D2_AFC0_B23731349F6B__INCLUDED_)
#define AFX_UTILS_H__0A7298A9_87BA_11D2_AFC0_B23731349F6B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <string>
#include <math.h>
#include "MSU_AFX.h"

namespace MSU
{


inline unsigned char double2char(double x)
{
	int p = int(x+0.5);
	return (p<=0) ? (unsigned char)0 :
		((p>=255) ? (unsigned char)255 : (unsigned char)p);
}

inline int double2int(double x)
{
	return int(x+0.5);
}

inline std::string int2string(int i, int radix=10 )
{
	char buffer[20];   
	_itoa( i, buffer, radix );
	return std::string(buffer);
}

inline double sqr(double x)
{
	return x*x;
}


inline double length(const CPoint &pt1, const CPoint &pt2)
{
	return sqrt(sqr(pt1.x-pt2.x) + sqr(pt1.y-pt2.y));
}



// declaration of initialization block
#define BEGIN_INITIALIZATION(f) \
	struct _Init_##f \
	{\
		_Init_##f(); \
	};\
	static _Init_##f _OInit;\
	inline _Init_##f::_Init_##f()\
	{


#define END_INITIALIZATION() \
	}


// declaration of finalization block
#define BEGIN_FINALIZATION(f) \
	struct _Final_##f \
	{\
		~_Final_##f(); \
	};\
	static _Final_##f _OFinal;\
	inline _Final_##f::~_Final_##f()\
	{

#define END_FINALIZATION() \
	}



// declaration of initialization+finalization block
#define BEGIN_INIT_FINAL(f) \
	struct _InitFinal_##f \
	{\
		_InitFinal_##f(); \
		~_InitFinal_##f(); \
	};\
	static _InitFinal_##f _OInitFinal;

#define INITIALIZATION(f)\
	} \
	inline _InitFinal_##f::_InitFinal_##f()\
	{

#define FINALIZATION(f) \
	} \
	inline _InitFinal_##f::~_InitFinal_##f()\
	{

#define END_INIT_FINAL() \
	}


#define shift_of(Class, Member) (int)&(((Class*)(0))->Member)



class EMSUException
{
public:
	EMSUException(const char* msg): m_msg(msg) {}
	EMSUException(const std::string &msg): m_msg(msg) {}
	const std::string &Message() { return m_msg; }
	void Delete() { delete this; }
protected:
	std::string m_msg;
};


#define MSU_THROW(msg) \
	throw new EMSUException(std::string(msg) + "\n  file: " + __FILE__ + "\n  line: " + int2string(__LINE__))

// process application message and 
// returns bTerminated=true if the application has received WM_QUIT (i.e. all calculations should be finished)
void ProcessAppMessages(bool& bTerminated); 

// returns bTerminated
bool ProcessAppMessages(); 



} // namespace MSU


#endif // !defined(AFX_UTILS_H__0A7298A9_87BA_11D2_AFC0_B23731349F6B__INCLUDED_)
